#include SFCryptor.H

* Declare the functions that we'll use in XICrCore.

lcDLL = 'XICrCore.DLL'
declare integer CRYIni_Initialize      in (lcDLL) integer LoadMode
declare integer CRYIni_UnInitialize    in (lcDLL)
declare integer CRYUtl_EncodeString    in (lcDLL) string strSrc, ;
	string @ strDest, integer dwLength, string strPassword, integer dwMethod
declare integer CRYUtl_DecodeString    in (lcDLL) string strSrc, ;
	string @ strDest, integer dwLength, string strPassword, integer dwMethod
declare string  CRYUtl_GetErrorMessage in (lcDLL) integer errorCode
declare integer CRYUtl_Encode          in (lcDLL) string strFilename, ;
	string strPassword, string strBackupExt, integer bKeepBackup, ;
	integer dwMethod
declare integer CRYUtl_Decode          in (lcDLL) string strFilename, ;
	string strPassword, string strBackupExt, integer bKeepBackup, ;
	integer dwMethod
declare integer CRYMan_Register        in (lcDLL) string strFilename, ;
	string strPassword, integer dwFlags, integer dwMethod
declare integer CRYMan_Unregister      in (lcDLL) string strFilename

* Initialize Cryptor.

lnStatus = CRYIni_Initialize(CRYPTOR_LOADMODE_NORMAL)
if not inlist(lnStatus, CRYPTOR_ERR_SUCCESS, CRYPTOR_ERR_ALREADY_INITIALIZED)
	messagebox('Could not initialize Cryptor: status code ' + ;
		transform(lnStatus))
	return
endif not inlist(lnStatus ...

* Encrypt a string.

lcStringToEncrypt = 'MyString'
lnLen             = len(lcStringToEncrypt)
lcEncryptedString = space(lnLen)
lcPassword        = 'FoxRocks!'
lnMethod          = ENCODER_LEVEL2	&& try other methods to see the difference
lnStatus          = CRYUtl_EncodeString(lcStringToEncrypt, ;
	@lcEncryptedString, lnLen, lcPassword, lnMethod)
if lnStatus = CRYPTOR_ERR_SUCCESS
	messagebox(lcStringToEncrypt + ' encrypts using method ' + ;
		transform(lnMethod) + ' and password ' + lcPassword + ' to ' + ;
		lcEncryptedString)

* Decrypt the same string.

	lcStringToEncrypt = space(lnLen)
	lnStatus          = CRYUtl_DecodeString(lcEncryptedString, ;
		@lcStringToEncrypt, lnLen, lcPassword, lnMethod)
	if lnStatus = CRYPTOR_ERR_SUCCESS
		messagebox('The encrypted string decrypts back to ' + ;
			lcStringToEncrypt)
	else
		messagebox('Could not decrypt string: ' + ;
			CRYUtl_GetErrorMessage(lnStatus))
	endif lnStatus = CRYPTOR_ERR_SUCCESS
else
	messagebox('Could not encrypt string: ' + ;
		CRYUtl_GetErrorMessage(lnStatus))
endif lnStatus = CRYPTOR_ERR_SUCCESS

* Let's check out the file encode and decode functions.

create table TEST (FIELD1 C(10), MEMO1 M)
index on FIELD1 tag FIELD1
insert into TEST values ('A', 'My memo')
use
lcFile   = fullpath('TEST.DBF')
lnStatus = CRYUtl_Encode(lcFile, lcPassword, .NULL., 0, lnMethod)
if lnStatus = CRYPTOR_ERR_SUCCESS
	messagebox('We should get an error when trying to USE the table ' + ;
		"because it's encrypted. Choose Ignore so we can continue.")
	use TEST
	lnStatus = CRYUtl_Decode(fullpath('TEST.DBF'), lcPassword, .NULL., 0, ;
		lnMethod)
	if lnStatus = CRYPTOR_ERR_SUCCESS
		use TEST
		if used('TEST')
			messagebox('The table was successfully decrypted.')
			browse
			use
		endif used('TEST')
	else
		messagebox('Could not decrypt table: ' + ;
			CRYUtl_GetErrorMessage(lnStatus))
	endif lnStatus = CRYPTOR_ERR_SUCCESS
else
	messagebox('Could not encrypt table: ' + ;
		CRYUtl_GetErrorMessage(lnStatus))
endif lnStatus = CRYPTOR_ERR_SUCCESS

* Now let's see how we can decrypt the file in memory only so it stays
* encrypted on disk.

lnStatus = CRYUtl_Encode(lcFile, lcPassword, .NULL., 0, lnMethod)
lnStatus = CRYUtl_Encode(forceext(lcFile, 'cdx'), lcPassword, .NULL., 0, ;
	lnMethod)
lnStatus = CRYUtl_Encode(forceext(lcFile, 'fpt'), lcPassword, .NULL., 0, ;
	lnMethod)
if lnStatus = CRYPTOR_ERR_SUCCESS
	lnStatus = CRYMan_Register(forceext(lcFile, '*'), lcPassword, 0, lnMethod)
* Can also register DBF, FPT, and CDX separately
endif lnStatus = CRYPTOR_ERR_SUCCESS
if inlist(lnStatus, CRYPTOR_ERR_SUCCESS, CRYPTOR_ERR_ALREADY_REGISTERED)
	use TEST
	if used('TEST')
		messagebox('The table was decrypted in memory so we can use it.')
		browse
		use
	endif used('TEST')
	lnStatus = CRYMan_Unregister(forceext(lcFile, '*'))
	if lnStatus = CRYPTOR_ERR_SUCCESS
		messagebox('We should get an error when trying to USE the table ' + ;
			"because it's still encrypted. Choose Ignore so we can continue.")
		use TEST
	else
		messagebox('Could not unregister table: ' + ;
			CRYUtl_GetErrorMessage(lnStatus))
	endif lnStatus = CRYPTOR_ERR_SUCCESS
else
	messagebox('Could not register table: ' + ;
		CRYUtl_GetErrorMessage(lnStatus))
endif inlist(lnStatus ...

* Uninitialize Cryptor and clean up.

lnStatus = CRYIni_UnInitialize()
if lnStatus <> CRYPTOR_ERR_SUCCESS
	messagebox('Could not uninitialize Cryptor: ' + ;
		CRYUtl_GetErrorMessage(lnStatus))
endif lnStatus <> CRYPTOR_ERR_SUCCESS
erase TEST.DBF
erase TEST.CDX
erase TEST.FPT
